@@ -1,14 +1,14 @@
-import pickle
-import warnings
-from scipy import stats
-from math import sqrt
-from sklearn import linear_model
-from abc import ABCMeta, abstractmethod
+from geosoup.raster import Raster, np, gdal_array, gdal
+from geosoup.common import Handler, Opt
 from sklearn.ensemble import RandomForestRegressor
 from sklearn.model_selection import GridSearchCV
 from sklearn.metrics import mean_squared_error
-from geosoup.common import Handler, Opt
-from geosoup.raster import Raster, np, gdal_array, gdal
+from sklearn import linear_model
+from abc import ABCMeta, abstractmethod
+from scipy import stats
+from math import sqrt
+import warnings
+import pickle
 
 
 __all__ = ['HRFRegressor',
@@ -57,21 +57,21 @@ class _Regressor(object, metaclass=ABCMeta):
                  use_weights=False):
         """
         Train the regressor
-        :param data: dictionary with values (generated using Samples.format_data())
+        :param data: Samples object
         :param use_weights: If the sample weights provided should be used? (default: False)
-        :return: Nonetype
+        :return: None
         """
         self.data = data
 
         if self.regressor is not None:
 
-            if 'weights' not in data or not use_weights:
-                self.regressor.fit(data['features'], data['labels'])
+            if (data.weights is None) or (not use_weights):
+                self.regressor.fit(data.x, data.y)
             else:
-                self.regressor.fit(data['features'], data['labels'], data['weights'])
+                self.regressor.fit(data.x, data.y, data.weights)
 
-        self.features = data['feature_names']
-        self.label = data['label_name']
+        self.features = data.x_name
+        self.label = data.y_name
         self.fit = True
 
     def pickle_it(self,
@@ -101,7 +101,7 @@ class _Regressor(object, metaclass=ABCMeta):
         """
         Find out how well the training samples fit the model
         :param data: Samples() instance
-        :param output_type: Metric to be omputed from the random forest (options: 'mean','median','sd')
+        :param output_type: Metric to be computed from the random forest (options: 'mean','median','sd')
         :param regress_limit: List of upper and lower regression limits for training fit prediction
         :return: None
         """
@@ -125,14 +125,14 @@ class _Regressor(object, metaclass=ABCMeta):
             self.training_results['intercept'] = pred['intercept']
             self.training_results['rmse'] = pred['rmse']
         else:
-            raise ValueError("Model not initialized with samples")
+            raise ValueError("Model not initialized with samples. Use fit_data() method")
 
     @abstractmethod
     def predict(self, *args, **kwargs):
         """
         Placeholder of subclass predict() methods
         """
-        pass
+        return
 
     @staticmethod
     def param_grid(param_dict):
@@ -179,7 +179,7 @@ class _Regressor(object, metaclass=ABCMeta):
         results = []
 
         for trn_data, test_data in folds:
-            regressor.fit_data(trn_data.format_data(),
+            regressor.fit_data(trn_data,
                                use_weights=use_weights)
 
             get_adjustment_param = getattr(regressor, 'get_adjustment_param', None)
@@ -188,7 +188,7 @@ class _Regressor(object, metaclass=ABCMeta):
                 get_adjustment_param(data_limits=regress_limit,
                                      output_type=output_type)
 
-            pred = regressor.sample_predictions(test_data.format_data(),
+            pred = regressor.sample_predictions(test_data,
                                                 regress_limit=regress_limit,
                                                 output_type=output_type)
             results.append(pred)
@@ -222,7 +222,7 @@ class _Regressor(object, metaclass=ABCMeta):
         """
         Method to search optimal parameters for the regressor. If 'select' is True, then this method
         will return a set of parameters with the best model score.
-        :param data: dictionary with values (generated using Samples.format_data())
+        :param data: Samples object
         :param param_dict: Dictionary of parameter grid to search for best score
         :param cv_folds: number of folds to divide the samples in
         :param n_jobs: number of parallel processes to run the grid search
@@ -235,21 +235,21 @@ class _Regressor(object, metaclass=ABCMeta):
             raise RuntimeError('Regressor not defined!')
 
         self.data = data
-        self.features = data['feature_names']
-        self.label = data['label_name']
+        self.features = data.x_name
+        self.label = data.y_name
 
         model = GridSearchCV(self.regressor,
                              param_dict,
                              cv=cv_folds,
                              n_jobs=n_jobs)
 
-        if 'weights' not in data:
-            model.fit(data['features'],
-                      data['labels'])
+        if data.weights is None:
+            model.fit(data.x,
+                      data.y)
         else:
-            model.fit(data['features'],
-                      data['labels'],
-                      sample_weight=data['weights'])
+            model.fit(data.x,
+                      data.y,
+                      sample_weight=data.weights)
 
         results = model.cv_results_
 
@@ -323,8 +323,7 @@ class _Regressor(object, metaclass=ABCMeta):
                 'band_additives': np.array([defaults['array_additive'] for _ in raster.bnames]),
                 'tile_size': min([nrows, ncols]) ** 2 if (min([nrows, ncols]) ** 2) <= defaults['tile_size']
                 else defaults['tile_size'],
-                'out_nodatavalue': kwargs['nodatavalue'] if defaults['out_nodatavalue']
-                is None else defaults['out_nodatavalue'],
+                'out_nodatavalue': kwargs['nodatavalue'] if 'nodatavalue' in kwargs else defaults['out_nodatavalue'],
                 'check_bands': check_bands,
             }
 
@@ -373,7 +372,7 @@ class _Regressor(object, metaclass=ABCMeta):
                        outfile=None,
                        outdir=None,
                        output_type='mean',
-                       band_name='band_1',
+                       band_name=None,
                        **kwargs):
         """
         Tree variance from the RF regressor
@@ -408,11 +407,14 @@ class _Regressor(object, metaclass=ABCMeta):
         if not raster_obj.init:
             raster_obj.initialize(nan_replacement=nodatavalue)
 
+        if band_name is None:
+            band_name = regressor.label
+
         kwargs.update({'out_data_type':
-                       gdal_array.NumericTypeCodeToGDALTypeCode(regressor.data['labels'].dtype)})
+                       gdal_array.NumericTypeCodeToGDALTypeCode(regressor.data.y.dtype)})
 
         defaults = regressor.get_defaults(raster=raster_obj,
-                                           **kwargs)
+                                          **kwargs)
 
         verbose = defaults['verbose'] if 'verbose' in defaults else False
         defaults['verbose'] = False
@@ -587,7 +589,7 @@ class MRegressor(_Regressor):
                  **kwargs):
         """
         Instantiate MRegressor class
-        :param data:  Data as Samples() instance
+        :param data:  Samples object
         :param regressor: Linear regressor
         :param fit_intercept: Whether to calculate the intercept for this model (default: True)
         :param n_jobs: The number of jobs to use for the computation
@@ -737,28 +739,28 @@ class MRegressor(_Regressor):
                            **kwargs):
         """
         Get predictions from the multiple regressor
-        :param data: Dictionary object from Samples.format_data
+        :param data: Samples object
         """
         if 'verbose' in kwargs:
             verbose = kwargs['verbose']
         else:
             verbose = False
 
-        self.feature_index = list(data['feature_names'].index(feat) for feat in self.features)
+        self.feature_index = list(data.x_name.index(feat) for feat in self.features)
 
         # calculate variance of tree predictions
-        y = self.predict(data['features'],
+        y = self.predict(data.x,
                          verbose=verbose)
 
         # rms error of the predicted versus actual
-        rmse = sqrt(mean_squared_error(data['labels'], y))
+        rmse = sqrt(mean_squared_error(data.y, y))
 
         # r-squared of predicted versus actual
-        lm = self.linear_regress(data['labels'], y)
+        lm = self.linear_regress(data.y, y)
 
         return {
             'pred': y,
-            'labels': data['labels'],
+            'labels': data.y,
             'rmse': rmse,
             'rsq': lm['rsq'],
             'slope': lm['slope'],
@@ -778,7 +780,7 @@ class MRegressor(_Regressor):
         :return: None
         """
         if data_limits is None:
-            data_limits = [self.data['labels'].min(), self.data['labels'].max()]
+            data_limits = [self.data.y.min(), self.data.y.max()]
 
         regress_limit = [data_limits[0] + clip * (data_limits[1]-data_limits[0]),
                          data_limits[1] - clip * (data_limits[1]-data_limits[0])]
@@ -868,7 +870,8 @@ class RFRegressor(_Regressor):
                                                                     ', '.join(self.features)))
 
             if attr_truth[2]:
-                print_str_list.append("Output: {}\n".format(self.regressor.n_outputs_))
+                print_str_list.append("Output: {} : {} \n".format(self.regressor.n_outputs_,
+                                                                  self.label))
 
             if attr_truth[3]:
                 print_str_list.append("OOB Score: {:{w}.{p}f} %".format(self.regressor.oob_score_ * 100.0,
@@ -906,7 +909,7 @@ class RFRegressor(_Regressor):
         """
 
         if min_variance is None:
-            min_variance = 0.025 * (self.data['labels'].max() - self.data['labels'].min())
+            min_variance = 0.025 * (self.data.y.max() - self.data.y.min())
 
         # List of index of bands to be used for regression
         if 'feature_index' in kwargs:
@@ -1209,7 +1212,7 @@ class RFRegressor(_Regressor):
         """
 
         return [(band, importance) for band, importance in
-                zip(self.data['feature_names'], self.regressor.feature_importances_)]
+                zip(self.data.x_name, self.regressor.feature_importances_)]
 
     def sample_predictions(self,
                            data,
@@ -1217,7 +1220,7 @@ class RFRegressor(_Regressor):
                            **kwargs):
         """
         Get tree predictions from the RF regressor
-        :param data: Dictionary object from Samples.format_data
+        :param data: Samples object
         :param output_type: Metric to be computed from RandomForestRegressor.
                            (options: 'mean','median','sd', 'var','full')
         :param kwargs: Keyword arguments:
@@ -1240,32 +1243,32 @@ class RFRegressor(_Regressor):
         else:
             verbose = False
 
-        self.feature_index = list(data['feature_names'].index(feat) for feat in self.features)
+        self.feature_index = list(data.x_name.index(feat) for feat in self.features)
 
         if 'regress_limit' in kwargs:
             regress_limit = kwargs['regress_limit']
         else:
             regress_limit = None
 
-        prediction = self.predict(data['features'],
+        prediction = self.predict(data.x,
                                   output_type=output_type,
                                   verbose=verbose)
 
         if regress_limit is not None:
-            lm = self.linear_regress(data['labels'],
+            lm = self.linear_regress(data.y,
                                      prediction,
                                      xlim=regress_limit)
         else:
-            lm = self.linear_regress(data['labels'],
+            lm = self.linear_regress(data.y,
                                      prediction)
 
-        rmse = sqrt(mean_squared_error(data['labels'], prediction))
+        rmse = sqrt(mean_squared_error(data.y, prediction))
 
         # if outfile and pickle file are not provided,
         # then only return values
         out_dict = {
             'pred': prediction,
-            'labels': data['labels'],
+            'labels': data.y,
             'rmse': rmse,
             'rsq': lm['rsq'],
             'slope': lm['slope'],
@@ -1288,7 +1291,7 @@ class RFRegressor(_Regressor):
         """
 
         if data_limits is None:
-            data_limits = [self.data['labels'].min(), self.data['labels'].max()]
+            data_limits = [self.data.y.min(), self.data.y.max()]
 
         regress_limit = [data_limits[0] + clip * (data_limits[1]-data_limits[0]),
                          data_limits[0] + (1.0-clip) * (data_limits[1]-data_limits[0])]
