@@ -138,7 +138,7 @@ class Samples:
                     if any(weights_colname in n for n in self.x_name):
                         loc = self.x_name.index(weights_colname)
                     else:
-                        raise ValueError("Weight column name mismatch.\nAvailable names: " + ', '.join(temp['name']))
+                        raise ValueError("Weight column name mismatch")
 
                     self.weights = self.x[:, loc]
                     self.x = np.delete(self.x, loc, 1)
@@ -211,36 +211,37 @@ class Samples:
         if self.csv_file is not None:
             return "<Samples object from {cf} with {v} variables, {n} samples>".format(cf=Handler(
                                                                                        self.csv_file).basename,
-                                                                                       n=len(self.x),
-                                                                                       v=len(self.x_name))
+                                                                                       n=self.x.shape[0],
+                                                                                       v=self.x.shape[1])
         elif self.csv_file is None and self.x is not None:
-            return "<Samples object with {n} samples>".format(n=len(self.x))
+            return "<Samples object with {v} variables, {n} samples>".format(n=self.x.shape[0],
+                                                                             v=self.x.shape[1])
         else:
-            return "<Samples object: __empty__>"
+            return "<Samples object: EMPTY>"
 
     def subsample(self,
-                  item):
+                  index_locations):
         """
         Method to get index locations as a sample object
-        :param item: list, tuple, numpy array or integer of index locations
+        :param index_locations: list, tuple, numpy array or integer of index locations
         :returns: Sample object
         """
-        if isinstance(item, list) or \
-                isinstance(item, tuple) or \
-                isinstance(item, np.ndarray) or \
-                isinstance(item, int):
+        if isinstance(index_locations, list) or \
+                isinstance(index_locations, tuple) or \
+                isinstance(index_locations, np.ndarray) or \
+                isinstance(index_locations, int):
 
             outsamp = Samples()
             outsamp.x_name = self.x_name
             outsamp.y_name = self.y_name
 
-            if isinstance(item, int):
-                item = np.array([item])
+            if isinstance(index_locations, int):
+                loc = np.array([index_locations])
             else:
-                item = np.array(item)
+                loc = np.array(index_locations)
 
-            outsamp.x = self.x[np.array(item), :]
-            outsamp.y = self.y[np.array(item)]
+            outsamp.x = self.x[np.array(loc), :]
+            outsamp.y = self.y[np.array(loc)]
 
             outsamp.nsamp = outsamp.x.shape[0]
             outsamp.index = np.arange(0, outsamp.nsamp)
@@ -645,4 +646,3 @@ class Samples:
                 self.x_hist[:, dim], self.x_bin_edges[:, dim] = np.histogram(self.x[:, dim], bins=nbins_x)
         else:
             warnings.warn('Flexible data type for X - Cannot compute histograms')
-
